/* Copyright 2009 Jetro Lauha - http://iki.fi/jetro/
 *
 * $Id: MainState.h 65 2009-08-30 21:58:25Z tonic $
 * $Revision: 65 $
 */

#ifndef MAINSTATE_H_INCL
#define MAINSTATE_H_INCL


#include <vector>

//using namespace std;
using namespace irr;
using namespace scene;
using namespace core;
using namespace video;


class MainState : public AppState
{
public:

    MainState();
    virtual ~MainState();

    // From AppState:

    virtual bool init();
    virtual void deinit();

    virtual bool update(UI32 time);
    virtual void render(irr::IrrlichtDevice *device);

    virtual void onActivate(UI32 time);
    virtual void onDeactivate() {}

    virtual bool onKeyDown(const irr::SEvent::SKeyInput &event);
    virtual bool onKeyUp(const irr::SEvent::SKeyInput &event);
    virtual bool onMouseDown(const irr::SEvent::SMouseInput &event);
    virtual bool onMouseUp(const irr::SEvent::SMouseInput &event);
    virtual bool onMouseMotion(const irr::SEvent::SMouseInput &event);
    virtual bool onMouseWheel(const irr::SEvent::SMouseInput &event);
    // Irr is lacking focus lost/gain events..
    //virtual void onFocusLost(const SDL_ActiveEvent &event) {}
    //virtual void onFocusGain(const SDL_ActiveEvent &event) {}

protected:

    enum IMAGE_ID
    {
        IMG_BUTTON = 0,
        IMG_BUTTON_HOT,
        IMG_BUTTON_ACTIVE,
        IMG_BUTTON_HOT_ACTIVE,
        IMG_CAVE_CIRCLE,
        IMG_SHIP,
        IMG_SHIP_BORDERLINE,
        IMG_FLARE,
        IMG_FLAME
    };

    enum KEY_ID
    {
        KID_LEFT = 0,
        KID_RIGHT,
        KID_UP,
        KID_DOWN,
        KID_COUNT //
    };

    struct LevelCell
    {
        enum {
            LCF_EMPTY = 1,
            LCF_MOVING = 2,
            LCF_NO_COLLIDER = 4,
            LCF_GOLD = 8
        };
        float x, y;
        float r;
        video::SColor color;
        int flags;
    };

    void loadLevel();

    void genCone(float x, float y, float r, const video::SColor &color = video::SColor(0xffffffff));

    App *mApp;

    IMGUI mIMGUI;
    IMGUIImageButtonStyle *mButtonStyle;

    Font *mFont;
    std::vector<Image *> mImages;
    C8 *mLabelText;

    IrrlichtDevice *mDevice;
    IVideoDriver *mVideo;
    ISceneManager *mSmgr;

    irr::core::array<irr::u16> mIndices;
    irr::core::array<irr::video::S3DVertex> mVertices;

    std::vector<LevelCell> mLevelCells;
    int mLevelWidth, mLevelHeight;  // cell grid

    ICameraSceneNode *mCamera;

    vector2di mLastMousePos;
    bool mKey[KID_COUNT];

    int mThrustChannel;
    int mDigChannel;
    int mDigCounter;

    bool mTitleUIActive;
    bool mInGame;
    bool mEndingUIActive;

    core::vector2df mPlayerPos;
    core::vector2df mPlayerSpeed;

    double mCash;

    int mTreasureCount;
    float mCamAngle;
};


#endif // !MAINSTATE_H_INCL
